@@ -47,9 +47,16 @@ enum
   GCONF_PROP_SELECTED_EFFECTS,
   GCONF_PROP_X_RESOLUTION,
   GCONF_PROP_Y_RESOLUTION,
+  GCONF_PROP_BRIGHTNESS,
+  GCONF_PROP_CONTRAST,
+  GCONF_PROP_SATURATION,
+  GCONF_PROP_HUE,
   GCONF_PROP_VIDEO_PATH,
   GCONF_PROP_PHOTO_PATH,
-  GCONF_PROP_ENABLE_DELETE
+  GCONF_PROP_ENABLE_DELETE,
+  GCONF_PROP_WIDE_MODE,
+  GCONF_PROP_BURST_DELAY,
+  GCONF_PROP_BURST_REPEAT
 };
 
 GType        cheese_gconf_get_type (void);
