@@ -39,7 +39,6 @@
 
 G_DEFINE_TYPE (CheeseThumbView, cheese_thumb_view, GTK_TYPE_ICON_VIEW);
 
-
 typedef struct
 {
   GtkListStore *store;
@@ -536,9 +535,7 @@ cheese_thumb_view_fill (CheeseThumbView *thumb_view)
 static void
 cheese_thumb_view_finalize (GObject *object)
 {
-  CheeseThumbView *thumb_view;
-
-  thumb_view = CHEESE_THUMB_VIEW (object);
+  CheeseThumbView *thumb_view = CHEESE_THUMB_VIEW (object);
   CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
 
   g_object_unref (priv->store);
@@ -568,7 +565,6 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
   char *path_videos = NULL, *path_photos = NULL;
 
   GFile    *file;
-  const int THUMB_VIEW_HEIGHT = 120;
 
   eog_thumbnail_init ();
 
@@ -578,7 +574,9 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
 
   gtk_icon_view_set_model (GTK_ICON_VIEW (thumb_view), GTK_TREE_MODEL (priv->store));
 
-  gtk_widget_set_size_request (GTK_WIDGET (thumb_view), -1, THUMB_VIEW_HEIGHT);
+  gtk_icon_view_set_margin (GTK_ICON_VIEW (thumb_view), 0);
+  gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (thumb_view), 0);
+  gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (thumb_view), 0);
 
   path_videos = cheese_fileutil_get_video_path (priv->fileutil);
   path_photos = cheese_fileutil_get_photo_path (priv->fileutil);
@@ -607,11 +605,8 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
   }
 
   gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (thumb_view), 0);
-#ifdef HILDON
-  gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), -1);
-#else
+
   gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), G_MAXINT);
-#endif
 
   gtk_icon_view_enable_model_drag_source (GTK_ICON_VIEW (thumb_view), GDK_BUTTON1_MASK,
                                           target_table, G_N_ELEMENTS (target_table),
