@@ -43,10 +43,6 @@
   #include <X11/XF86keysym.h>
 #endif /* HAVE_MMKEYS */
 
-#ifdef HILDON
-  #include <hildon/hildon-program.h>
-#endif
-
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
 #include "cheese-fileutil.h"
@@ -64,13 +60,14 @@
 #define FULLSCREEN_POPUP_HEIGHT    40
 #define FULLSCREEN_TIMEOUT         5 * 1000
 #define FULLSCREEN_EFFECTS_TIMEOUT 15
-#define DEFAULT_WINDOW_WIDTH       600
-#define DEFAULT_WINDOW_HEIGHT      450
+#define DEFAULT_WINDOW_WIDTH       520
+#define DEFAULT_WINDOW_HEIGHT      390
 
 typedef enum
 {
   WEBCAM_MODE_PHOTO,
-  WEBCAM_MODE_VIDEO
+  WEBCAM_MODE_VIDEO,
+  WEBCAM_MODE_BURST
 } WebcamMode;
 
 typedef enum
@@ -113,6 +110,11 @@ typedef struct
   GtkWidget *main_vbox;
   GtkWidget *video_vbox;
 
+  GtkWidget *netbook_alignment;
+  GtkWidget *toolbar_alignment;
+  GtkWidget *effect_button_alignment;
+  GtkWidget *togglegroup_alignment;
+
   GtkWidget *effect_frame;
   GtkWidget *effect_alignment;
   GtkWidget *effect_chooser;
@@ -128,9 +130,11 @@ typedef struct
   GtkWidget *button_effects;
   GtkWidget *button_photo;
   GtkWidget *button_video;
+  GtkWidget *button_burst;
   GtkWidget *button_effects_fullscreen;
   GtkWidget *button_photo_fullscreen;
   GtkWidget *button_video_fullscreen;
+  GtkWidget *button_burst_fullscreen;
   GtkWidget *button_exit_fullscreen;
 
   GtkWidget *image_take_photo;
@@ -152,11 +156,6 @@ typedef struct
   GtkWidget *take_picture;
   GtkWidget *take_picture_fullscreen;
 
-#ifdef HILDON
-  GtkWidget *main_hbox;
-  GtkWidget *subwindow;
-#endif
-
   GtkActionGroup *actions_account_photo;
   GtkActionGroup *actions_countdown;
   GtkActionGroup *actions_effects;
@@ -170,7 +169,9 @@ typedef struct
   GtkActionGroup *actions_photo;
   GtkActionGroup *actions_toggle;
   GtkActionGroup *actions_video;
+  GtkActionGroup *actions_burst;
   GtkActionGroup *actions_fullscreen;
+  GtkActionGroup *actions_wide_mode;
 
   GtkUIManager *ui_manager;
 
@@ -178,6 +179,10 @@ typedef struct
 
   int audio_play_counter;
 
+  gint repeat_count;
+  gboolean is_bursting;
+  gboolean needs_resizing;
+
   CheeseFlash *flash;
 } CheeseWindow;
 
@@ -319,6 +324,134 @@ cheese_window_fullscreen_motion_notify_cb (GtkWidget      *widget,
   return FALSE;
 }
 
+static void
+cheese_window_queue_size_negotiation (CheeseWindow *cheese_window)
+{
+  gtk_widget_queue_resize_no_redraw (GTK_WIDGET (cheese_window->thumb_view));
+  cheese_window->needs_resizing = TRUE;
+}
+
+static void
+cheese_window_window_size_req_cb (GtkWidget      *widget,
+                                  GtkRequisition *req,
+                                  CheeseWindow   *cheese_window)
+{
+  /* resize to minimum allowed by current constraints */
+  gtk_window_resize (GTK_WINDOW (widget), req->width, req->height);
+  g_signal_handlers_disconnect_by_func (widget, G_CALLBACK (cheese_window_window_size_req_cb), cheese_window);
+  /* remove constraints */
+  gtk_widget_set_size_request (cheese_window->notebook, -1, -1);
+}
+
+static void
+cheese_window_thumb_view_size_req_cb (GtkWidget      *widget,
+                                      GtkRequisition *req,
+                                      CheeseWindow   *cheese_window)
+{
+  /* every time this shit runs a sweet lovely puppy dies somewhere in
+   * the world, please spend some time and save them! */
+  /* the thing is: how to keep the icon view to a minimum size without
+     setting a permanent size request? */
+  if ((req->width == 0) && (req->height == 0)) {
+    gtk_widget_set_size_request (widget, 140, 100);
+  } else {
+    gtk_widget_set_size_request (widget, -1, -1);
+  }
+
+  /* set scrollbar policy. Can't be done in the
+   * eog_thumb_nav_set_vertical because the icon view relayouts on
+   * idle after changing the "column" property. So if you set NEVER
+   * policy before the relayout the window gets as bigger as many
+   * items you've got */
+  if (eog_thumb_nav_is_vertical (EOG_THUMB_NAV (cheese_window->thumb_nav)))
+  {
+    eog_thumb_nav_set_policy (EOG_THUMB_NAV (cheese_window->thumb_nav),
+                              GTK_POLICY_NEVER,
+                              GTK_POLICY_AUTOMATIC);
+  } else {
+    eog_thumb_nav_set_policy (EOG_THUMB_NAV (cheese_window->thumb_nav),
+                              GTK_POLICY_AUTOMATIC,
+                              GTK_POLICY_NEVER);
+  }
+
+  /* at this time the toplevel window has still no size requisition,
+   * wait for its next size-request */
+  if (cheese_window->needs_resizing) {
+    g_signal_connect (G_OBJECT (cheese_window->window),
+                      "size-request",
+                      G_CALLBACK (cheese_window_window_size_req_cb),
+                      cheese_window);
+    cheese_window->needs_resizing = FALSE;
+  }
+}
+
+static void
+cheese_window_toggle_wide_mode (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  gboolean toggled = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (widget));
+
+  /* 1. set a size request on the notebook to keep the video area size
+   *    while switching mode. So it feels like the thumbview is moving
+   *    from the bottom to the left (well it actually is..)
+   *
+   * 2. trigger a resize on the thumbview and set needs_resizing
+   *
+   * 3. wait for next icon view size-request, at this time the icon view
+   *    has reached its definitive size and we can:
+   *    - resize the window to its minimum requisition
+   *    - remove size contraints on the netbook
+   *
+   * This eventually works for setting default window size at startup
+   * too since it prevents the video screen to shrink down without any
+   * need of a permanent size request
+   */
+  gtk_widget_set_size_request (cheese_window->notebook, GTK_WIDGET
+                               (cheese_window->notebook)->allocation.width,
+                               GTK_WIDGET (cheese_window->notebook)->allocation.height);
+  cheese_window_queue_size_negotiation (cheese_window);
+
+  /* set a single column in wide mode */
+  gtk_icon_view_set_columns (GTK_ICON_VIEW (cheese_window->thumb_view), toggled ? 1 : G_MAXINT);
+  /* switch thumb_nav mode */
+  eog_thumb_nav_set_vertical (EOG_THUMB_NAV (cheese_window->thumb_nav), toggled);
+  /* reparent thumb_view */
+  g_object_ref (cheese_window->thumb_scrollwindow);
+  if (toggled) {
+    gtk_container_remove (GTK_CONTAINER (cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
+    gtk_container_add (GTK_CONTAINER (cheese_window->netbook_alignment), cheese_window->thumb_scrollwindow);
+    g_object_unref (cheese_window->thumb_scrollwindow);
+  } else {
+    gtk_container_remove (GTK_CONTAINER (cheese_window->netbook_alignment), cheese_window->thumb_scrollwindow);
+    gtk_box_pack_end (GTK_BOX (cheese_window->video_vbox), cheese_window->thumb_scrollwindow, FALSE, FALSE, 0);
+    g_object_unref (cheese_window->thumb_scrollwindow);
+  }
+
+  /* update spacing */
+  /* NOTE: be really carefull when changing the ui file to update spacing
+   * values here too! */
+  if (toggled) {
+    g_object_set (G_OBJECT (cheese_window->toolbar_alignment),
+                  "bottom-padding", 10, NULL);
+    g_object_set (G_OBJECT (cheese_window->togglegroup_alignment),
+                  "left-padding", 6, NULL);
+    g_object_set (G_OBJECT (cheese_window->effect_button_alignment),
+                  "right-padding", 0, NULL);
+    g_object_set (G_OBJECT (cheese_window->netbook_alignment),
+                  "left-padding", 6, NULL);
+  } else {
+    g_object_set (G_OBJECT (cheese_window->toolbar_alignment),
+                  "bottom-padding", 6, NULL);
+    g_object_set (G_OBJECT (cheese_window->togglegroup_alignment),
+                  "left-padding", 24, NULL);
+    g_object_set (G_OBJECT (cheese_window->effect_button_alignment),
+                  "right-padding", 24, NULL);
+    g_object_set (G_OBJECT (cheese_window->netbook_alignment),
+                  "left-padding", 0, NULL);
+  }
+
+  g_object_set (cheese_window->gconf, "gconf_prop_wide_mode", toggled, NULL);
+}
+
 static void
 cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -396,8 +529,12 @@ static void
 cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
   gdk_threads_enter ();
-  gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
-  gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  if (!cheese_window->is_bursting) {
+    gtk_action_group_set_sensitive (cheese_window->actions_effects, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_toggle, TRUE);
+    gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
+    gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  }
   gdk_flush ();
   gdk_threads_leave ();
 }
@@ -436,6 +573,7 @@ cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
   g_object_unref (cheese_window->actions_preferences);
   g_object_unref (cheese_window->actions_file);
   g_object_unref (cheese_window->actions_video);
+  g_object_unref (cheese_window->actions_burst);
   g_object_unref (cheese_window->actions_fullscreen);
   g_object_unref (cheese_window->gconf);
 
@@ -1146,6 +1284,10 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
     {
       gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
     }
+    else if (cheese_window->webcam_mode == WEBCAM_MODE_BURST)
+    {
+      gtk_action_group_set_sensitive (cheese_window->actions_burst, TRUE);
+    }
     else
     {
       gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
@@ -1163,6 +1305,7 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
     gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   }
 }
 
@@ -1280,53 +1423,90 @@ cheese_window_escape_key_cb (CheeseWindow *cheese_window,
   return TRUE;
 }
 
-static void
-cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
+static gboolean
+cheese_window_take_photo (gpointer data)
 {
-  char *str;
+  gboolean      countdown;
+  CheeseWindow *cheese_window = (CheeseWindow *) data;
 
-  if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
+  g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+  if (countdown)
   {
-    gboolean countdown;
-    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
-    if (countdown)
+    if (cheese_window->isFullscreen)
     {
-      if (cheese_window->isFullscreen)
-      {
-        cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown_fullscreen,
-                                cheese_window_countdown_picture_cb,
-                                cheese_window_countdown_hide_cb,
-                                (gpointer) cheese_window);
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->fullscreen_bar), 1);
+      cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown_fullscreen,
+                              cheese_window_countdown_picture_cb,
+                              cheese_window_countdown_hide_cb,
+                              (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->fullscreen_bar), 1);
 
-        /* show bar, start timeout
-         * ATTENTION: if the countdown is longer than FULLSCREEN_TIMEOUT,
-         * the bar will disapear before the countdown ends
-         */
-        cheese_window_fullscreen_show_bar (cheese_window);
-        cheese_window_fullscreen_set_timeout (cheese_window);
-      }
-      else
-      {
-        cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown,
-                                cheese_window_countdown_picture_cb,
-                                cheese_window_countdown_hide_cb,
-                                (gpointer) cheese_window);
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook_bar), 1);
-      }
+      /* show bar, start timeout
+       * ATTENTION: if the countdown is longer than FULLSCREEN_TIMEOUT,
+       * the bar will disapear before the countdown ends
+       */
+      cheese_window_fullscreen_show_bar (cheese_window);
+      cheese_window_fullscreen_set_timeout (cheese_window);
     }
     else
     {
-      cheese_window_countdown_picture_cb (cheese_window);
+      cheese_countdown_start ((CheeseCountdown *) cheese_window->countdown,
+                              cheese_window_countdown_picture_cb,
+                              cheese_window_countdown_hide_cb,
+                              (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook_bar), 1);
     }
-
-    gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
-    gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, FALSE);
-
-    /* FIXME: set menu inactive */
   }
-  else if (cheese_window->webcam_mode == WEBCAM_MODE_VIDEO)
+  else
   {
+    cheese_window_countdown_picture_cb (cheese_window);
+  }
+
+  gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
+  gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, FALSE);
+
+  if (cheese_window->webcam_mode == WEBCAM_MODE_BURST) {
+    guint repeat_delay = 1000;
+    gboolean countdown = FALSE;
+
+    g_object_get (cheese_window->gconf, "gconf_prop_burst_delay", &repeat_delay, NULL);
+    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+
+    if (countdown && repeat_delay < 5000)
+    {
+      /* A countdown takes 4 seconds, leave some headroom before repeating it.
+       * Magic number chosen via expiriment on Netbook */
+      repeat_delay = 5000;
+    }
+    /* start burst mode phot series */
+    if (!cheese_window->is_bursting) {
+      g_timeout_add (repeat_delay, cheese_window_take_photo, cheese_window);
+      cheese_window->is_bursting = TRUE;
+    }
+    cheese_window->repeat_count--;
+    if (cheese_window->repeat_count > 0)
+    {
+      return TRUE;
+    }
+  }
+  cheese_window->is_bursting = FALSE;
+
+  return FALSE;
+}
+
+static void
+cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  char     *str;
+
+  switch (cheese_window->webcam_mode) {
+  case WEBCAM_MODE_BURST:
+    gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_toggle, FALSE);
+    g_object_get (cheese_window->gconf, "gconf_prop_burst_repeat", &cheese_window->repeat_count, NULL); /* reset burst counter */
+  case WEBCAM_MODE_PHOTO:
+    cheese_window_take_photo (cheese_window);
+    break;
+  case WEBCAM_MODE_VIDEO:
     if (!cheese_window->recording)
     {
       gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
@@ -1351,6 +1531,10 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
     {
       cheese_window_stop_recording (cheese_window);
     }
+    break;
+  default:
+    g_assert_not_reached ();
+    break;
   }
 }
 
@@ -1392,9 +1576,14 @@ static const GtkToggleActionEntry action_entries_fullscreen[] = {
   {"Fullscreen", GTK_STOCK_FULLSCREEN, NULL, "F11", NULL, G_CALLBACK (cheese_window_toggle_fullscreen), FALSE},
 };
 
+static const GtkToggleActionEntry action_entries_wide_mode[] = {
+  {"WideMode", NULL, N_("_Wide mode"), NULL, NULL, G_CALLBACK (cheese_window_toggle_wide_mode), FALSE},
+};
+
 static const GtkRadioActionEntry action_entries_toggle[] = {
   {"Photo", NULL, N_("_Photo"), NULL, NULL, 0},
   {"Video", NULL, N_("_Video"), NULL, NULL, 1},
+  {"Burst", NULL, N_("_Burst"), NULL, NULL, 2},
 };
 
 static const GtkActionEntry action_entries_file[] = {
@@ -1409,13 +1598,17 @@ static const GtkActionEntry action_entries_file[] = {
 };
 
 static const GtkActionEntry action_entries_photo[] = {
-  {"TakePhoto", NULL, N_("_Take a photo"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
+  {"TakePhoto", NULL, N_("_Take a Photo"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
 };
 
 static const GtkToggleActionEntry action_entries_video[] = {
   {"TakeVideo", NULL, N_("_Recording"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb), FALSE},
 };
 
+static const GtkActionEntry action_entries_burst[] = {
+  {"TakeBurst", NULL, N_("_Take multiple Photos"), "space", NULL, G_CALLBACK (cheese_window_action_button_clicked_cb)},
+};
+
 static const GtkActionEntry action_entries_account_photo[] = {
   {"SetAsAccountPhoto", NULL, N_("_Set As Account Photo"), NULL, NULL, G_CALLBACK (cheese_window_cmd_set_about_me_photo)},
 };
@@ -1452,6 +1645,20 @@ cheese_window_activate_radio_action (GtkAction *action, GtkRadioAction *current,
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
+  }
+  else if (strcmp (gtk_action_get_name (GTK_ACTION (current)), "Burst") == 0)
+  {
+    cheese_window->webcam_mode = WEBCAM_MODE_BURST;
+
+    str = g_strconcat ("<b>", _("_Take multiple Photos"), "</b>", NULL);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), g_strdup (str));
+    gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo_fullscreen), g_strdup (str));
+    gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
   }
   else
   {
@@ -1464,6 +1671,7 @@ cheese_window_activate_radio_action (GtkAction *action, GtkRadioAction *current,
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo_fullscreen), TRUE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   }
   g_free (str);
 }
@@ -1517,7 +1725,7 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
 
 static void
 cheese_window_set_info_bar (CheeseWindow *cheese_window,
-                                GtkWidget    *info_bar)
+                            GtkWidget    *info_bar)
 {
   if (cheese_window->info_bar == info_bar)
     return;
@@ -1540,15 +1748,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   GError     *error = NULL;
   char       *path;
   GtkBuilder *builder;
-
-#ifdef HILDON
-  HildonProgram *program = hildon_program_get_instance ();
-  GtkWidget     *menu;
-  GtkWidget     *menuitem;
-#else
   GtkWidget *menubar;
-#endif
-
 
   cheese_window->info_bar = NULL;
 
@@ -1565,12 +1765,17 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->button_effects              = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects"));
   cheese_window->button_photo                = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo"));
   cheese_window->button_video                = GTK_WIDGET (gtk_builder_get_object (builder, "button_video"));
+  cheese_window->button_burst                = GTK_WIDGET (gtk_builder_get_object (builder, "button_burst"));
   cheese_window->image_take_photo            = GTK_WIDGET (gtk_builder_get_object (builder, "image_take_photo"));
   cheese_window->label_effects               = GTK_WIDGET (gtk_builder_get_object (builder, "label_effects"));
   cheese_window->label_photo                 = GTK_WIDGET (gtk_builder_get_object (builder, "label_photo"));
   cheese_window->label_take_photo            = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
   cheese_window->label_video                 = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
   cheese_window->main_vbox                   = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->netbook_alignment           = GTK_WIDGET (gtk_builder_get_object (builder, "netbook_alignment"));
+  cheese_window->togglegroup_alignment       = GTK_WIDGET (gtk_builder_get_object (builder, "togglegroup_alignment"));
+  cheese_window->effect_button_alignment     = GTK_WIDGET (gtk_builder_get_object (builder, "effect_button_alignment"));
+  cheese_window->toolbar_alignment           = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar_alignment"));
   cheese_window->video_vbox                  = GTK_WIDGET (gtk_builder_get_object (builder, "video_vbox"));
   cheese_window->notebook                    = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
   cheese_window->notebook_bar                = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
@@ -1587,6 +1792,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->button_effects_fullscreen   = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects_fullscreen"));
   cheese_window->button_photo_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo_fullscreen"));
   cheese_window->button_video_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_video_fullscreen"));
+  cheese_window->button_burst_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "button_burst_fullscreen"));
   cheese_window->take_picture_fullscreen     = GTK_WIDGET (gtk_builder_get_object (builder, "take_picture_fullscreen"));
   cheese_window->label_take_photo_fullscreen =
     GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo_fullscreen"));
@@ -1598,6 +1804,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
     GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame_fullscreen"));
   cheese_window->button_exit_fullscreen = GTK_WIDGET (gtk_builder_get_object (builder, "button_exit_fullscreen"));
 
+  g_object_unref (builder);
+
   /* configure the popup position and size */
   GdkScreen *screen = gtk_window_get_screen (GTK_WINDOW (cheese_window->fullscreen_popup));
   gtk_window_set_default_size (GTK_WINDOW (cheese_window->fullscreen_popup),
@@ -1614,30 +1822,6 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                     G_CALLBACK (cheese_window_exit_fullscreen_button_clicked_cb),
                     cheese_window);
 
-#ifdef HILDON
-  /* Reparent widgets in case we use hildon. This saves us maintaining two
-   * GtkBuilder ui files
-   */
-  cheese_window->window    = hildon_window_new ();
-  cheese_window->main_hbox = gtk_hbox_new (FALSE, 0);
-  hildon_program_add_window (program, HILDON_WINDOW (cheese_window->window));
-  gtk_container_add (GTK_CONTAINER (cheese_window->window), cheese_window->main_hbox);
-  g_object_ref (cheese_window->thumb_scrollwindow);
-  g_object_ref (cheese_window->video_vbox);
-  gtk_container_remove (GTK_CONTAINER (cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
-  gtk_container_remove (GTK_CONTAINER (cheese_window->main_vbox), cheese_window->video_vbox);
-  gtk_box_pack_start (GTK_BOX (cheese_window->main_hbox), cheese_window->video_vbox, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (cheese_window->main_hbox), GTK_WIDGET (cheese_window->thumb_scrollwindow),
-                      FALSE, FALSE, 0);
-  gtk_widget_destroy (cheese_window->main_vbox);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cheese_window->thumb_scrollwindow),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  g_object_unref (cheese_window->thumb_scrollwindow);
-  g_object_unref (cheese_window->video_vbox);
-#endif
-
-  g_object_unref (builder);
-
   char *str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo_fullscreen), str);
@@ -1648,7 +1832,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), FALSE);
 
   cheese_window->thumb_view = cheese_thumb_view_new ();
-  cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, TRUE);
+  cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, FALSE);
+
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_nav);
 
   /* show the scroll window to get it included in the size requisition done later */
@@ -1674,9 +1859,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
 
   gtk_widget_realize (cheese_window->screen);
   GdkWindow *win = gtk_widget_get_window (cheese_window->screen);
-  if (!gdk_window_ensure_native (win)) {
+  if (!gdk_window_ensure_native (win))
+  {
     /* FIXME: this breaks offscreen stuff, we should really find
      * another way to embed video that doesn't require an XID */
+
     /* abort: no native window, no xoverlay, no cheese. */
     g_error ("Could not create a native X11 window for the drawing area");
   }
@@ -1705,6 +1892,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                              action_entries_fullscreen,
                                                                              G_N_ELEMENTS (action_entries_fullscreen));
 
+  cheese_window->actions_wide_mode = cheese_window_toggle_action_group_new (cheese_window,
+                                                                            "ActionsWideMode",
+                                                                            action_entries_wide_mode,
+                                                                            G_N_ELEMENTS (action_entries_fullscreen));
+
   cheese_window->actions_preferences = cheese_window_action_group_new (cheese_window,
                                                                        "ActionsPreferences",
                                                                        action_entries_preferences,
@@ -1726,6 +1918,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                         action_entries_video,
                                                                         G_N_ELEMENTS (action_entries_video));
   gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
+  cheese_window->actions_burst = cheese_window_action_group_new (cheese_window,
+                                                                 "ActionsBurst",
+                                                                 action_entries_burst,
+                                                                 G_N_ELEMENTS (action_entries_burst));
+  gtk_action_group_set_sensitive (cheese_window->actions_burst, FALSE);
   cheese_window->actions_account_photo = cheese_window_action_group_new (cheese_window,
                                                                          "ActionsAccountPhoto",
                                                                          action_entries_account_photo,
@@ -1793,25 +1990,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   g_object_get (cheese_window->gconf, "gconf_prop_enable_delete", &enable_delete, NULL);
   gtk_action_set_visible (GTK_ACTION (action), enable_delete);
 
-#ifdef HILDON
-  menu     = gtk_menu_new ();
-  menuitem = gtk_menu_item_new_with_label (_("Quit"));
-  g_signal_connect (menuitem, "activate",
-                    G_CALLBACK (cheese_window_cmd_close),
-                    cheese_window);
-  gtk_menu_shell_append (menu, menuitem);
-
-  menuitem = gtk_menu_item_new_with_label (_("About"));
-  g_signal_connect (menuitem, "activate",
-                    G_CALLBACK (cheese_window_cmd_about),
-                    cheese_window);
-  gtk_menu_shell_append (menu, menuitem);
-
-  hildon_window_set_menu (HILDON_WINDOW (cheese_window->window), GTK_MENU (menu));
-#else
   menubar = gtk_ui_manager_get_widget (cheese_window->ui_manager, "/MainMenu");
   gtk_box_pack_start (GTK_BOX (cheese_window->main_vbox), menubar, FALSE, FALSE, 0);
-#endif
 
   cheese_window->thumb_view_popup_menu = gtk_ui_manager_get_widget (cheese_window->ui_manager,
                                                                     "/ThumbnailPopup");
@@ -1837,6 +2017,10 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_video), action);
   gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_video_fullscreen), action);
 
+  action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/Burst");
+  gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_burst), action);
+  gtk_activatable_set_related_action (GTK_ACTIVATABLE (cheese_window->button_burst_fullscreen), action);
+
 
   /* Default handlers for closing the application */
   g_signal_connect (cheese_window->window, "destroy",
@@ -1960,9 +2144,10 @@ setup_camera (CheeseWindow *cheese_window)
 }
 
 void
-cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
+cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server, gboolean startup_in_wide_mode)
 {
   CheeseWindow *cheese_window;
+  gboolean startup_in_wide_mode_saved;
 
   cheese_window = g_new0 (CheeseWindow, 1);
 
@@ -1972,6 +2157,8 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->flash               = cheese_flash_new ();
   cheese_window->audio_play_counter  = 0;
   cheese_window->isFullscreen        = FALSE;
+  cheese_window->is_bursting         = FALSE;
+  cheese_window->needs_resizing      = FALSE;
 
   cheese_window->server = dbus_server;
 
@@ -1990,14 +2177,28 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
   cheese_window->recording   = FALSE;
 
+  g_object_get (cheese_window->gconf,
+                "gconf_prop_wide_mode",
+                &startup_in_wide_mode_saved,
+                NULL);
+
+  startup_in_wide_mode = startup_in_wide_mode_saved ? TRUE : startup_in_wide_mode;
+
+  if (startup_in_wide_mode) {
+    GtkAction *action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/WideMode");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+
   /* handy trick to set default size of the drawing area while not
-   * limiting its minimum size, thanks Owen! */
-  GtkRequisition req;
-  gtk_widget_set_size_request (cheese_window->screen,
+   * limiting its minimum size, thanks Owen! -- slightly modified, see
+   * comment in toggle_wide_mode to understand how it works -- */
+
+  g_signal_connect (cheese_window->thumb_view, "size-request",
+                    G_CALLBACK (cheese_window_thumb_view_size_req_cb),
+                    cheese_window);
+  gtk_widget_set_size_request (cheese_window->notebook,
                                DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
-  gtk_widget_size_request (cheese_window->window, &req);
-  gtk_window_set_default_size (GTK_WINDOW (cheese_window->window), req.width, req.height);
-  gtk_widget_set_size_request (cheese_window->screen, -1, -1);
+  cheese_window_queue_size_negotiation (cheese_window);
 
   gtk_widget_show_all (cheese_window->window);
 
